@@ -5,13 +5,14 @@ import time
 import gobject, dbus
 from dbus.mainloop.glib import DBusGMainLoop
 import sys
-import evolution
 import os.path
 import subprocess
 import shlex
 import codecs
 import traceback
 import logging
+import sys
+
 
 
 cfgFileLoc = "~/.espeakcaller/"
@@ -26,100 +27,18 @@ useNickname = 0
 eSpeakProc = 0
 repeatTime = 5
 repeatEnabled = 1
+repeatCount = 5
 silenceSpeakers = 0
 eSpeakCommand = ""
 
 _HEADPHONE_SYS = "/sys/devices/platform/gpio-switch/headphone/state"
 
-def get_name_from_number(number):
-    #  Taken from fMMS (modified)
-    #  Should get caller name from phonebook
-    #  I wonder if there is better/faster/less dirty way to do this...
-    #
-    # Fixed (0.3): "eSpeakCaller says the number even if the name is defined"-bug :)
-    #              (number is in international format)
-    # Thanks to Thnninen @ http://talk.maemo.org/showthread.php?t=34982&page=4
-    #
-    # Fixed: (0.4): "eSpeakCaller still has no clue about callername"-bug
-    #              (number contains spaces)
-    #
-    # Fixed: (0.5): "eSpeakCaller doesn't understand callernumber with brackets()"-bug
-    #
-    
-    try:
-        if (len(number) == 0) :
-            return "%blocked%"
-    
-        ab = evolution.ebook.open_addressbook("default")
-        contacts = ab.get_all_contacts()
-        for c in contacts:
-            vcardlist = c.get_vcard_string().replace('\r', '').split('\n')
-            fname = ""
-            tmpNick = ""
-            tmpFName = ""
-            tmpLName = ""
-            nr = ""
-            my_num = ""
-            c_num = ""
-            tmpId = c.get_uid()
-            
-            for line in vcardlist:    
-                if line.startswith("TEL"):
-                    nr = line.split(":")[1]
-                                 
-                    my_num = number.lstrip('0')      # Remove leading zero
-                    my_num = my_num.replace(' ', '') # Remove spaces 
-                    my_num = my_num.replace('(', '') # Remove brackets '()'
-                    my_num = my_num.replace(')', '')
-                                    
-                    c_num = nr.lstrip('0')
-                    c_num = c_num.replace(' ', '')
-                    c_num = c_num.replace('(', '') 
-                    c_num = c_num.replace(')', '')
-                    
-                    if c_num.endswith(my_num) or my_num.endswith(c_num):
-                        fname = c.get_name()
-                        if fname != "" and fname != None:
-                            Log("Found name: " + fname)
-                            Log("Phonebook number: " + nr)
-                            Log("Phonebook vcard: " + "\n".join(vcardlist))
-                            Log("Stripped incoming number: " + my_num)
-                            Log("Stripped phonebook number: " + c_num)
-                    #end if
-                #end if
-                if line.startswith("NICKNAME"):
-                    tmpNick = line.split(":")[1]
-                if line.startswith("N:") or line.startswith("N;"):
-                    tmpStr = line.split(":")[1]
-                    if len(tmpStr.split(";"))>1:          
-                        tmpFName = tmpStr.split(";")[1]
-                        tmpLName = tmpStr.split(";")[0]
-                    else:
-                        tmpFName = ""
-                        tmpLName = tmpStr
-                    #end if
-                #end if
-            #next
-            
-            if fname != "" and fname != None: 
-                Log("First name: " + tmpFName)
-                Log("Last name: " + tmpLName)
-               
-                return [fname.decode('utf-8'), tmpFName.decode('utf-8'), tmpLName.decode('utf-8'), tmpNick.decode('utf-8'), tmpId]
-    
-            #end if
-        #next
-        
-        # Number not found in phonebook.
-        return "%number%"
-    except Exception, err:
-        logger.exception("Error in get_name_from_number()")
-        pass
-        
 
 #Check if headphones are connected.
 def headphones():
-    state = open(_HEADPHONE_SYS, "r").read().strip()
+    HeadphoneSys = open(_HEADPHONE_SYS, "r")
+    state = HeadphoneSys.read().strip()
+    HeadphoneSys.close
     if (state == "disconnected"):
         #print "Headphones disconnected"
         return False
@@ -138,11 +57,15 @@ def _amixer(name, value):
 # Incoming phonecall...
 def handle_call(obj_path, callernumber):
     global eSpeakCommand
-   
+    global muteTest
+    
     try:    
         Log("Incoming phonecall from number: " + callernumber)
-        CallerNames = get_name_from_number(callernumber)
-    
+        import espeakcaller_contacts
+
+        #CallerNames = get_name_from_number(callernumber)
+        CallerNames = espeakcaller_contacts.get_name_from_number(str(callernumber))
+        #print CallerNames
         eSpeakText = ""
         
         if CallerNames == "%number%": # Number not in phonebook :(
@@ -161,6 +84,7 @@ def handle_call(obj_path, callernumber):
                 if Enabled == 1:           
                     eSpeakText = sayName
         else: #we got lucky and found name from phonebook :)
+            Log("Found name: " + CallerNames[0])
             loadsettings(str(CallerNames[4]))
             if Enabled == 1:
                 if sayName.find("%nick%") != -1 and CallerNames[3] != "":  # Use nickname if available and wanted.
@@ -175,14 +99,13 @@ def handle_call(obj_path, callernumber):
                     eSpeakText = eSpeakText.replace("%last%", CallerNames[2])      
         
         if eSpeakText != "":
-            # Fixed: (0.4) "me speaks no umlauts"-bug (use utf-8)
-            # Added: Repeat option. (just quick hack really)
+            # Added (0.4): Repeat option. (just quick hack really, but seems to do the trick)
             Log("Say text: " + eSpeakText)
-            if repeatEnabled == 1:
-                eSpeakText = (eSpeakText + '<break time="' + str(repeatTime) + 's"/>') * 10 # max repeat 10 times (enough?)
+            
+            eSpeakText = (eSpeakText + '<break time="' + str(repeatTime) + 's"/>') * repeatCount 
             
             eSpeakText = '<break time="2s"/>' + eSpeakText       
-            eSpeakCommand = unicode(myCommand.replace("%text%", eSpeakText)).encode("utf-8")
+            eSpeakCommand = myCommand.replace("%text%", eSpeakText).encode("utf-8")
 
     except Exception, err:
         logger.exception("Error in handle_call()")
@@ -198,27 +121,34 @@ def get_profile():
 # Fixed: (0.5) "eSpeak starts shouting things after end ring"-bug
 # Do things based on call status.
 def call_status(state,ukn1,ukn2):
-    #Log("state: %d  ukn1: %d  ukn2: %d" % (state,ukn1,ukn2))
+    Log("state: %d  ukn1: %d  ukn2: %d" % (state,ukn1,ukn2))
+    #print "state: %d  ukn1: %d  ukn2: %d" % (state,ukn1,ukn2)
     global eSpeakProc
     global eSpeakCommand
-
+    
     try:
         if state == 5: # STATUS_MT_ALERTING
+            
             if eSpeakCommand != "":
-                args = shlex.split(eSpeakCommand)
-                #Fixed: (0.6) Be quiet when in silent profile :)
-                if get_profile() != "silent": 
+                
+                #Fixed: (0.6) Be quiet when in silent profile. :)
+                if get_profile() != "silent":
                     Log("Calling eSpeak. Command: " + eSpeakCommand)
+                    args = shlex.split(eSpeakCommand)
                     eSpeakProc = subprocess.Popen(args, shell=False)
                     # Fixed: (0.6) Silence speakers if using headphones (dirty hack?)
+                    # TODO: Support for bluetooth headphones?
                     if headphones() and silenceSpeakers == 1: 
-                        _amixer("Speaker Function", "0")        
+                        Log("Using headphones. Disabling speakers")
+                        _amixer("Speaker Function", "0")  
+                    StartMonitor()    
+                         
                 eSpeakCommand = ""
-        elif state == 10 or state == 15 or state == 7: # STATUS_TERMINATED or STATUS_ANSWERED
-            eSpeakCommand = ""
-            if headphones() and silenceSpeakers == 1: 
-                _amixer("Speaker Function", "1")  
-            subprocess.Popen("kill -9 " + str(eSpeakProc.pid), shell=True)
+        elif state == 10 or state == 15 or state == 7 or state == 9 or state == 0: # STATUS_TERMINATED or STATUS_ANSWERED
+            StopMonitor()
+            if eSpeakProc != 0:
+                subprocess.Popen("kill -9 " + str(eSpeakProc.pid), shell=True)
+                eSpeakProc = 0
     except Exception, err:
         logger.exception("Error in call_status()")
         pass
@@ -236,13 +166,11 @@ def loadsettings(contact):
                 #print "notfound " + cfgTmp
                 cfgTmp = cfgFileLoc + cfgGeneral 
                 
-                         
         cfgTmp = os.path.expanduser(cfgTmp)
                
         if os.path.exists(cfgTmp): 
-            Log("Loaded config: " + cfgTmp)
+            Log("Loading config: " + cfgTmp)
      
-
             pdFile = codecs.open(cfgTmp, 'r','utf-8')
             lines = pdFile.readlines()
             pdFile.close()
@@ -251,9 +179,10 @@ def loadsettings(contact):
             global myCommand
             global sayName
             global useNickname
-            global repeatEnabled
+            global repeatCount
             global repeatTime
             global silenceSpeakers
+            global speakInSilentProfile
             
             for line in lines:
                 line = line.strip()
@@ -266,16 +195,56 @@ def loadsettings(contact):
                 elif line.startswith("Nick:"):
                     useNickname = int(line.split(":")[1])
                 elif line.startswith("Repeat:"):
-                    repeatEnabled = int(line.split(":")[1])
+                    if line.split(":")[1] == "1":
+                        repeatCount = 5
+                    else:
+                        repeatCount = 1
+                elif line.startswith("RepeatCount:"):
+                    repeatCount = int(line.split(":")[1])
                 elif line.startswith("Repeattime:"):
                     repeatTime = int(line.split(":")[1])
                 elif line.startswith("Headphones:"):
-                    silenceSpeakers = int(line.split(":")[1])                                
+                    silenceSpeakers = int(line.split(":")[1]) 
+                                                   
     except Exception, err:
         logger.exception("Error in loadsettings()")
         pass
         
-
+        
+# Fixed (0.7): Possible fix for "eSpeak is yelling after mute button has been pressed."-bug
+#              Reeeaaally dirty hack :b I still haven't found any real way to do this :(
+def StartMonitor():
+    try:
+        bus = dbus.SessionBus()
+        bus.add_match_string("")
+        bus.add_message_filter(msg_filter)
+    except:
+        pass
+def StopMonitor():
+    try:
+        bus = dbus.SessionBus()
+        bus.remove_message_filter(msg_filter)
+        bus.remove_match_string("")              
+    except:
+        pass
+def msg_filter(abus, msg):
+    try:
+        args = msg.get_args_list()
+        dest = msg.get_destination()
+        mem = msg.get_member()
+        
+        if dest == "org.maemo.Playback.Manager" and mem == "RequestState":
+            if args[1] == "Stop":
+                #print "Stop event!"
+                global eSpeakProc
+                if eSpeakProc != 0:   
+                    Log("Mute pressed. Killing espeak.")
+                    subprocess.Popen("kill -9 " + str(eSpeakProc.pid), shell=True)
+                    eSpeakProc = 0
+    except Exception, e:
+        logger.exception("Error in msg_filter()")
+     
+     
 def Log(logText):
     logger.debug(logText)
     
@@ -289,6 +258,7 @@ try:
     bus = dbus.SystemBus()
     bus.add_signal_receiver(handle_call, path='/com/nokia/csd/call', dbus_interface='com.nokia.csd.Call', signal_name='Coming')
     bus.add_signal_receiver(call_status, path=None, dbus_interface='com.nokia.csd.Call.Instance', signal_name='CallStatus')
+    
     gobject.MainLoop().run()
 except Exception, err:
     logger.exception("Error in main()")
